/* 
   This file is dedicated to error integration on the localization side 
   according to the paper odometer.tex to be found in the docs directory 
   It is supposed to compute and maintain the global error on Localization side
   
*/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "mat.h"
/* Variables */

#include "odometer.h" 


/* For the following variables : 
		     		     
extern double Loc2Abs[3];
extern double Vloc2abs[6];

extern double Glob_pos[4];
extern double Vpos[6];
		     
* OK this is not very nice but it allows for a smooth transition from *
*  the old names to something nicer ... *

extern double rob_enc[4];
extern double Vrob_enc[6];
extern double Prev_pos[4];
*/

/* Previous encoder position at which the last Localization was computed */
static double *PreviousEncoderPosition = NULL ;
/* Previous error computed either by the robot or by err_sim.c */
static double PreviousEncoderVariance [6]; 

/* Current encoder position corresponding to the position at which the robot 
   got its current laser data */
/*double CurrentEncoderPosition [3] ;*/
static double *CurrentEncoderPosition = NULL; 

/* Current error computed either by the robot or by err_sim.c */
static double  *CurrentEncoderVariance =NULL; 

/* local variables */

/* jacobian of CurrentEncoderPosition with respect to PreviousEncoderPosition */ 
static double J_CurrentEncPos_PreviousEncPos [9] = {1.0,0.0,0.0,
						    0.0, 1.0, 0.0,
						    0.0,0.0,1.0};

/* jacobian of Loc2Abs (should be called CurrentTransform) with respect to PreviousEncoderPosition */ 
static double J_Loc2Abs_PreviousEncPos [9] = {1.0,0.0,0.0,
					      0.0, 1.0, 0.0,
					      0.0,0.0,-1.0};


/* jacobian of Glob_pos (should be called CurrentLocalizationPosition) with respect to CurrentEncoderPosition */
 
static double J_Glob_pos_CurrentEncPos [9] = {1.0,0.0,0.0,
					      0.0, 1.0, 0.0,
					      0.0,0.0,1.0};

/* jacobian of Glob_pos (should be called CurrentLocalizationPosition) with respect to Loc2Abs */ 
static double J_Glob_pos_Loc2Abs [9] = {1.0,0.0,0.0,
					0.0, 1.0, 0.0,
					0.0,0.0,1.0};

/*auxilliaries */

static double aux[9];
static double aux2[9];
static double sn,cs;



void LinkExternalVariables()
{
  PreviousEncoderPosition = Prev_pos ;
  CurrentEncoderPosition = rob_enc; 
  CurrentEncoderVariance = Vrob_enc; 

}
/* 
   saving the previous encoder variance corresponding to the time of the last localization. 
   This should be called ONLY after a successful localization and NOT before 
   trying to localize.

*/

void SavePreviousEncoderVariance() {

  mxs_copy(CurrentEncoderVariance,PreviousEncoderVariance,3,3);

}

/* 
   computing the displacement between the CurrentEncoderPosition and PreviousEncoderPosition in
   CurrentEncoderPosition reference frame. Result in "move" 
   Useful for computing J_CurrentEncPos_PreviousEncPos.
   Well, maybe not, see below...
*/
/*
void LocalMoveBetweenPerception (double *move)
{



}
*/

/* 
   to be computed right before localizing (could be done only if 
   a solution has been found and before the final value is computed though)
   Well, if we look at J_CurrentEncPos_PreviousEncPos we see that the vector 
   [0][2], [1][2] is the rotation of the local move in x and y by the amount 
   of  PreviousEncoderPosition[2] + M_PI/2.
   Now, the local move is exactly the projection of CurrentEncoderPosition
   in  PreviousEncoderPosition reference frame, which means that the local
    move in x and y is the rotation of 
    (CurrentEncoderPosition[0] -PreviousEncoderPosition[0]) and 
    (CurrentEncoderPosition[1] - PreviousEncoderPosition[1]) by 
    the amount of -PreviousEncoderPosition[2]. 
   Then, the vector [0][2], [1][2] of  J_CurrentEncPos_PreviousEncPos is the rotation of (CurrentEncoderPosition[0] -PreviousEncoderPosition[0]) and (CurrentEncoderPosition[1] - PreviousEncoderPosition[1])  by the amount of +M_PI
   I guess there is no need to compute the local move after all :=)

*/

void Jac_CurrentEncPos_PreviousEncPos()
{  
  J_CurrentEncPos_PreviousEncPos[2] = -(CurrentEncoderPosition[1] - PreviousEncoderPosition[1]);
  J_CurrentEncPos_PreviousEncPos[5] = (CurrentEncoderPosition[0] - PreviousEncoderPosition[0]);

}



/* 
   to be computed right after a successful localization. 
   At this point, the PreviousEncoderPosition should  have been set to the CurrentEncoderPosition value.
   Or maybe I should still use the CurrentEncoderPosition value in case we want to try another 
   localization value ?? 
   OK, here it goes : the Previous value should be set when we get to the next localization and before doing anything else (ok, see set_encoders() in odom_sim.c) .
 
 */

void Jac_Loc2Abs_PreviousEncPos() {
  
  cs = cos(Loc2Abs[2]); sn = sin(Loc2Abs[2]);

  J_Loc2Abs_PreviousEncPos[0] = -cs; 
  J_Loc2Abs_PreviousEncPos[1] = sn; 
  J_Loc2Abs_PreviousEncPos[2] = -sn*CurrentEncoderPosition[0] - cs * CurrentEncoderPosition[1];   

  J_Loc2Abs_PreviousEncPos[3] = -sn; 
  J_Loc2Abs_PreviousEncPos[4] = -cs; 
  J_Loc2Abs_PreviousEncPos[5] = cs*CurrentEncoderPosition[0] - sn * CurrentEncoderPosition[1];   

}


void  Jac_Glob_pos_CurrentEncPos()
{
  cs = cos(Loc2Abs[2]); sn = sin(Loc2Abs[2]);
  
  J_Glob_pos_CurrentEncPos[0] = cs; 
  J_Glob_pos_CurrentEncPos[1] = -sn;
  
  
  J_Glob_pos_CurrentEncPos[3] = sn;
  J_Glob_pos_CurrentEncPos[4] = cs;


}

void Jac_Glob_pos_Loc2Abs ()
{
  cs = cos(Loc2Abs[2]); sn = sin(Loc2Abs[2]);
  J_Glob_pos_Loc2Abs[2] = -sn * CurrentEncoderPosition[0] - cs * CurrentEncoderPosition[1];
  J_Glob_pos_Loc2Abs[5] = cs * CurrentEncoderPosition[0] -sn * CurrentEncoderPosition[1];
  

}


/* Initialization at manual localization time */




/* computation at automatic localization time */

/**  XXXXX
 **  LocalizationPositionVariance - computation of the error after a move
 **
 ** Principle : add to last localisation uncertainties (ie, Vpos matrix)
 ** the uncertainties due to robot motion from last localization.
 **
 ** This motion uncertainty is deduced from the current uncertainty 
 ** of the odometer and the uncertainty of the odometer at last 
 ** localization. 
 ** 
 ** PROBLEM : THIS COMPUTATION IS NOT VALID IF FOR SOME EXTERNAL 
 **           REASON/PROCEDURE THE ODOMETER UNCERTAINTY HAS BEEN RE-SET
 **           (For instance, it will cause pb if we want to dynamically
 **            update the odometer uncertainty and/or position).
 **
 ** In such a case, Vpos should be just re-set also with the new Vrob_enc
 ** value (in the right reference).
 **
 ** Without any beter idea we at least test that the uncertainty matrix
 ** is coherent (ie, v[0], v[2] and v[5] > 0.0). If not, we keep the previous
 ** Vpos. But perhaps we should re-initialized Vpos....
 **
 ** Remark : There is certainly the same pb with the function
 ** 'integrate_error' that is used for robots without probabilistics 
 **  odometer (XR4000 for instance ...)
 **/

static double mxfs_det_3_3(double *mat);
int segreinit = 1;

/* computation of the error after a move */
void LocalizationPositionVariance()

{
  double saveVpos[6];
  double unused_pos[4];

  memcpy(saveVpos, Vpos, 6*sizeof(double));


  /* Test if encoder has not decreases (that would indicate that the odometer 
     has been re-set) */
  /* However the best test would be directly on the variance variation 
     but I do not know where it is computed in the computations below */
  if (mxfs_det_3_3(CurrentEncoderVariance) < 
      mxfs_det_3_3(PreviousEncoderVariance)) {
    printf ("LocalizationPositionVariance: det odo variance decreases !\n");
    /* Re-init Vpos with Vrob_enc */
    if (segreinit) {
      Calc_abs_pos(rob_enc,Vrob_enc,Loc2Abs,Vloc2abs,unused_pos,Vpos);
      return;
    }

  if (CurrentEncoderVariance[0] < PreviousEncoderVariance[0] ||
      CurrentEncoderVariance[2] < PreviousEncoderVariance[2] ||
      CurrentEncoderVariance[5] < PreviousEncoderVariance[5]) {
    printf ("LocalizationPositionVariance: odo variance decreases !\n");
    /* Re-init Vpos with Vrob_enc */
    if (segreinit) {
      Calc_abs_pos(rob_enc,Vrob_enc,Loc2Abs,Vloc2abs,unused_pos,Vpos);
      return;
    }
  }
  

  }
  
  /*first term of the big equation */

  Jac_Glob_pos_Loc2Abs();
  mxfs_mmulttr(J_Glob_pos_Loc2Abs,Vloc2abs,Vpos,3,3,3,3);  

  /*second term */
  Jac_Glob_pos_CurrentEncPos();
  mxfs_mmulttr(J_Glob_pos_CurrentEncPos,Vrob_enc,aux,3,3,3,3);
  mxs_plus(aux,Vpos,Vpos,3,3);

  /* correlation current-transform/curent-encoder-position */
  Jac_CurrentEncPos_PreviousEncPos();
  mxfs_mult(J_CurrentEncPos_PreviousEncPos,PreviousEncoderVariance,aux,3,3,3,3);
  mx_multtr(aux,J_Loc2Abs_PreviousEncPos,aux,3,3,3,3);
  
  /*third and fourth terms*/

  mx_mult(J_Glob_pos_CurrentEncPos,aux,aux,3,3,3,3);
  mx_multtr(aux,J_Glob_pos_Loc2Abs,aux,3,3,3,3);
  mx_plustr(aux,aux2,3,3);

  /*here we are ... I hope ! */ 

  mxs_plus(aux2,Vpos,Vpos,3,3);

  /* Coherency Test (see comment above) */
  if (Vpos[0] < 0.0 || Vpos[2] < 0.0 || Vpos[5] < 0.0) {
    printf ("LocalizationPositionVariance : var < 0.0 !\n");

    /* Keep previous value ... */
/*  XXXX   memcpy(Vpos, saveVpos, 6*sizeof(double)); */

    /* ... or Re-init Vpos with Vrob_enc */
    if (segreinit)
      Calc_abs_pos(rob_enc,Vrob_enc,Loc2Abs,Vloc2abs,unused_pos,Vpos);

  }

}

/* Ca doit bien exister quelque part cette fonction de calcul de 
   determinant de matrice ! */
static double mxfs_det_3_3(double *mat) 
{
  double det;
  det =   mat[0] * (mat[2] * mat[5] - mat[4] * mat[4])
	- mat[1] * (mat[1] * mat[5] - mat[4] * mat[3])
	+ mat[3] * (mat[1] * mat[4] - mat[2] * mat[3]);
  return det;
}






