
/*

  It is using the first section of odometer.tex now. Basically, for simulation, we decompose 
  the current position error into 
  - the projection of the old error along the LOCAL move. By local, we mean the movez expressed in 
  the old position frame
  - the error added by the local move. 

  This last one should be configurable in term of accuracy model . To be done 

*/


#include <stdio.h>
#include <math.h>
#include "err_pos_p.h"
#include "odometer.h"
#include "odometer_p.h"
#include "mat.h"
#include "lasutil.h"


/******
  These constants must be (re)initialized using 'SetAccuracyMove' function 
  (see below) according to robot's characteristics (currently done in 
  InitSick function in robots/sick_<robot>.c file).
  The following default values are for xr4000 in very good motion conditions.
******/

static double Vcom_trans = 0.005;
static double Vcom_rot = 0.005;

/* square of angle drift per meter */ 
static double rot_by_trans = 0.017 * 0.017; 

/*********
  see comments on integrate_error
**********/

static double Jprev[9] = {1.0, 0.0,0.0,
			 0.0,1.0,0.0,
			 0.0,0.0,1.0};



static double vcom[6] = {0.,0.,0.,0.,0.,0.};




/**************
  sets the accuracy model on the 
  robot position
  ec_lin = percent of the linear displacement:
           (straight line)
  ec_rot = percent of rotation.
  ec_ang_tr = radian / meter deviation
****************/


void SetAccuracyMove(double ec_lin, double ec_rot, double ec_ang_tr)

{
  Vcom_trans = (ec_lin*ec_lin);
  Vcom_rot = (ec_rot*ec_rot);
  rot_by_trans = ec_ang_tr * ec_ang_tr;
}


/*********************
  computes the new error variance from 
  - the previous error variance Vpos[6] (3*3 symetrical
    matrix)
    defined in ../include/rob_pos.h

  - the previous robot position computed
    in Glob_pos[4] defined in 
    defined in ../include/rob_pos.h

  - the command vector parameter c_trans and c_rot. 



  -1 : applies a model of uncertainty computing 
           the variance matrix Vcom[3] (2*2 symetrical
	   matrix); Update the static global 
	   variable Vcom;

       __Accuracy_model(c_trans,c_rot);

  -2 : computes the effect of the error due to the 
       command, assuming a perfect knowledge on the
       previous position of the robot : static global
       variable Vpos_com[6] (3*3).

       __Calc_Vpos_com(c_trans,c_rot)

  -3 : computes the effect on the new position  due
       to the previous error on the robot position
       assuming a perfect command: static global
       variable Vpos_prev[6] (3*3)
       
       __Calc_Vpos_prev(c_trans,c_rot)

  -4 : Adds the 2 components to computes Vrob_enc[6];
       __mxs_plus(Vpos_prev,Vpos_com,Vrob_enc,3,3);

***********************************/
 
void integrate_error(double c_trans,double c_rot)
{

  /*always allowing for 5 degrees */
  /*if(c_trans){
    if( c_rot < 0)
      c_rot -= 0.08;
    else
      c_rot += 0.08;
      }
  */

  
  vcom[0] = Vcom_trans * (rob_enc[0] - prev_odo[0]) * (rob_enc[0] - prev_odo[0]);
  vcom[2] = Vcom_trans * (rob_enc[1] - prev_odo[1]) * (rob_enc[1] - prev_odo[1]);
  
  vcom[5] = Vcom_rot * c_rot * c_rot + c_trans*c_trans * rot_by_trans;
  

  Jprev[2] = - (rob_enc[1] - prev_odo[1]);
  Jprev[5] = rob_enc[0] - prev_odo[0];
#ifdef VERBOSE
  printf("Pose av\n");
  mx_print(prev_odo,1,3);
  printf("Pose apres\n");
  mx_print(rob_enc,1,3);
#endif

  mxfs_mmulttr(Jprev,Vrob_enc,Vrob_enc,3,3,3,3);
  mxs_plus(Vrob_enc,vcom,Vrob_enc,3,3); 
#ifdef VERBOSE
  printf("Rajout a V_xrob\n");
  mx_print(vcom,3,3);
#endif

#ifdef VERBOSE
  printf ("dep x %5.4f y %5.4f t %5.4f d %5.4f\n"
	  "v0 %5.4f v1 %5.4f v2  %5.4f\n",
	  Jprev[5], -Jprev[2], c_rot, c_trans,
	  vcom[0], vcom[2], vcom[5]);
#endif

}



/************************
  Simulates Initializing the variance on the robot
*************************/

void Init_variance()

{
  
  Vrob_enc[0] = Vrob_enc[1] = Vrob_enc[2] = Vrob_enc[3] = Vrob_enc[4] = Vrob_enc[5] = 0.;


}














