#include <stdio.h>
#include <math.h>
#ifdef VXWORKS
#define M_PI 3.141592653589793
#endif

#include "odometer_p.h"
#include "odometer.h"
#include "mat.h"
#include "kal_prot.h"
#include "lasdef.h"
#include "lasutil_p.h"

/*********
  auxilliary for transform computing
*********/

static double Jo2a[9] = {1.,0.,0.,
			 0.,1.,0.,
			 0.,0.,1.};

static double Jr2a[9] = {1.,0.,0.,
			 0.,1.,0.,
			 0.,0.,1.};

static double Jt2r[9] = {1.,0.,0.,
			 0.,1.,0.,
			 0.,0.,-1.};

static double aux[9];


/**************
  compute the transformation from
  the absolute location and the robot encoders.

  Warning: abs_pos and tr must be different arrays

  NEW PHIL : (July 1999) adding the variance on the robot position there too. 
  Note : this prototype is very weird : if we need xrob (rob_enc in fact)  here we should need
  Vrob_enc as well .. This was because before we didn't integrate the robot error into the transform 
  variance. 
  
  In order  not to change too many things, I will ignore xrob here and always use rob_enc and Vrob_enc.
  TO BE DONE : xrob should be removed from here .

**************/

void CalcTransf(double *abs_pos,double *vabs_pos,double *xrob, double *tr,double *vtr)
{
  static double cloc2abs,sloc2abs;

  tr[2] = abs_pos[2] - rob_enc[2];

  cloc2abs = cos(tr[2]);
  sloc2abs = sin(tr[2]);

  tr[0] = abs_pos[0] - cloc2abs * rob_enc[0] + sloc2abs * rob_enc[1];
  tr[1] = abs_pos[1] - sloc2abs * rob_enc[0] - cloc2abs * rob_enc[1];



  /* computing the variance of the transform. Note that we do not 
   compute the covariance in err_integration.c yet as we didn't validate this hypothesis yet*/

  /********** 
     Jo2a  = 
     jacobian of the influence of abs_pos on tr value
  **********/

  Jo2a[2] = abs_pos[1] - tr[1];
  Jo2a[5] = tr[0] - abs_pos[0];

  mxfs_mmulttr(Jo2a,vabs_pos,vtr,3,3,3,3);


  /********** 
     Jt2r  = 
     jacobian of the influence of rob_enc on tr value
  **********/

  Jt2r[0] = -cloc2abs ; Jt2r[1] = sloc2abs; Jt2r[2] = - Jo2a[2];
  Jt2r[3] = -sloc2abs ; Jt2r[4] = -cloc2abs; Jt2r[5] = - Jo2a[5];

  mxfs_mmulttr(Jt2r,Vrob_enc,aux,3,3,3,3);

  /**********
    variance on the transformation
  **********/

  mxs_plus(vtr,aux,vtr,3,3);
  
  
  

}


/**************
  compute the transformation from
  the absolute location and the robot encoders.
  Same as above but without variance computation.

  Warning: abs_pos and tr must be different arrays
**************/

void CalcTransfNp(double *abs_pos,double *xrob,double *tr)
{
  static double cloc2abs,sloc2abs;

  tr[2] = abs_pos[2] - xrob[2];

  cloc2abs = cos(tr[2]);
  sloc2abs = sin(tr[2]);

  tr[0] = abs_pos[0] - cloc2abs * xrob[0] + sloc2abs * xrob[1];
  tr[1] = abs_pos[1] - sloc2abs * xrob[0] - cloc2abs * xrob[1];
    
}



void Calc_abs_pos(double *xrob,double *vrob,double *tr,double *vtr,double *abs,double *vabs)
{
  double  cloc2abs,sloc2abs;
  
  cloc2abs = cos(tr[2]);
  sloc2abs = sin(tr[2]);

  abs[0] =  cloc2abs * xrob[0] - sloc2abs * xrob[1] + tr[0]; 
  abs[1] =  sloc2abs * xrob[0] + cloc2abs * xrob[1] + tr[1];
  abs[2] =  norm_ang(xrob[2] + tr[2],-M_PI);
  abs[3] =  norm_ang(xrob[3] + tr[2],-M_PI);
  if(vtr){
  Jr2a[0] = cloc2abs; Jr2a[1] = -sloc2abs;
  Jr2a[3] = sloc2abs; Jr2a[4] = cloc2abs;

  Jo2a[2] = tr[1] - abs[1];
  Jo2a[5] = abs[0] - tr[0];

  mxfs_mmulttr(Jr2a,vrob,aux,3,3,3,3);
  mxfs_mmulttr(Jo2a,vtr,vabs,3,3,3,3);
  mxs_plus(aux,vabs,vabs,3,3);
  }
}



void Calc_abs_pos_np(double *xrob,double *tr,double *abs)
{
  double  cloc2abs,sloc2abs;
  
  cloc2abs = cos(tr[2]);
  sloc2abs = sin(tr[2]);

  abs[0] =  cloc2abs * xrob[0] - sloc2abs * xrob[1] + tr[0]; 
  abs[1] =  sloc2abs * xrob[0] + cloc2abs * xrob[1] + tr[1];
  abs[2] =  norm_ang(xrob[2] + tr[2],-M_PI);
  abs[3] =  norm_ang(xrob[3] + tr[2],-M_PI);

  
}

/****
  changes the coordinates of the robot and recompute the variance.
  recomputes the transform.
  This is mainly to recompute the variance 
  
  ****/

void RotateRobotCoordinates(double *dth,double *pos,double *vpos, double *enc, double *tr, double *vtr )
{ 

  double x,y;
  
  aux[0]= cos(dth[2]); aux[1] = -sin(dth[2]); aux[2] = 0;
  aux[3]= sin(dth[2]); aux[4] = cos(dth[2]); aux[5] = 0;
  aux[6]= 0; aux[7] = 0 ; aux[8] = 1;
  
  /* new position */
  x = pos[0]; 
  y = pos[1];
  pos[0] = cos(dth[2])*x - sin(dth[2])*y  + dth[0];
  pos[1] = cos(dth[2])*y + sin(dth[2])*x  + dth[1];
  pos[2] += dth[2];
  /* new variance */

  mxfs_mmulttr(aux,vpos,vpos,3,3,3,3);

  /* new transform */
  CalcTransf(pos,vpos, enc,tr,vtr);

}


/*Fonction for using kalman filtering in the case of identity. There are identity functions
defined in kalplus.c but they do not deal with angles which have to be normalized. 
The jacobian should be OK but the error fonction must take angle normalization into account*/


void pos_idt(double *y, double *x, double *r)
{
  r[0] = x[0] - y[0];
  r[1] = x[1] - y[1];
  r[2] = norm_ang(x[2]-y[2],-M_PI) ;
}

double pos_dist (double *xr1,double *vxr1,double *xr2,double *vxr2,double *xr3,double *vxr3){

  return Dist_maha(pos_idt,didty3,didtx3,xr1,vxr1,xr2,vxr2,NULL,xr3,vxr3,3,3,3);

}

/*

  merging 2 robot positions xr1, vxr1 and xr2,vxr2. result in xr3, vxr3 
  All matrices must be allocated by the user.
  If op : suppose the pos_dist function has been called with the same arguments right before.
  (NOT trhead safe !).

 

 */

void pos_merge (double *xr1,double *vxr1,double *xr2,double *vxr2,double *xr3,double *vxr3, int op)
{
  if(op){
      Dfusion();
  }
  else
    fusion(pos_idt,didty3,didtx3,xr1,vxr1,xr2,vxr2,NULL,xr3,vxr3,3,3,3);


}

/**
 **  Complementary test on jumps
 **
 **
 **
 **/

#define GAUSS_99 3.0 

static double pos_jump_filter_fact = 2.0;

void set_pos_jump_filter_fact(double fact)
{
  pos_jump_filter_fact = fact;
}

int pos_jump_filter(double GlobPos[4], double VGlobPos[6], 
		    double HypPos[4])
{
  double dx, dy, dt;
  double ux, uy, ut;

  dx = GlobPos[0] - HypPos[0];
  dy = GlobPos[1] - HypPos[1];
  dt = GlobPos[2] - HypPos[2];

  ux = sqrt(VGlobPos[0]) * GAUSS_99 * pos_jump_filter_fact;
  uy = sqrt(VGlobPos[2]) * GAUSS_99 * pos_jump_filter_fact;
  ut = sqrt(VGlobPos[5]) * GAUSS_99 * pos_jump_filter_fact;

  if (fabs(dx) > ux) {
#ifdef VERBOSE
    printf ("pos_jump_filter : jump dx %5.4f (ux %5.4f/%5.4f)\n", dx, ux, pos_jump_filter_fact);
#endif
    return 0;
  }
  if (fabs(dy) > uy) {
#ifdef VERBOSE
    printf ("pos_jump_filter : jump dy %5.4f (uy %5.4f/%5.4f)\n", dy, uy, pos_jump_filter_fact);
#endif
    return 0;
  }
  
  dt = norm_ang(dt,-M_PI);
  if (fabs(dt) > ut) {
#ifdef VERBOSE
    printf ("pos_jump_filter : jump dt %5.4f (ut %5.4f/%5.4f)\n", dt, ut, pos_jump_filter_fact);
#endif
    return 0;
  }
  return 1;
}

