#       $LAAS: robots.m4,v 1.1 2006/08/10 16:47:27 sara Exp $

#
# Copyright (c) 2002-2005 LAAS/CNRS                   --  Fri Mar 15 2002
# All rights reserved.
#
# Redistribution  and  use in source   and binary forms,  with or without
# modification, are permitted provided that  the following conditions are
# met:
#
#   1. Redistributions  of  source code must  retain  the above copyright
#      notice, this list of conditions and the following disclaimer.
#   2. Redistributions in binary form must  reproduce the above copyright
#      notice,  this list of  conditions and  the following disclaimer in
#      the  documentation   and/or  other  materials   provided with  the
#      distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE  AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY  EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES   OF MERCHANTABILITY AND  FITNESS  FOR  A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO  EVENT SHALL THE AUTHOR OR  CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING,  BUT  NOT LIMITED TO, PROCUREMENT  OF
# SUBSTITUTE  GOODS OR SERVICES;  LOSS   OF  USE,  DATA, OR PROFITS;   OR
# BUSINESS  INTERRUPTION) HOWEVER CAUSED AND  ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE  USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
dnl --- Look for the laas mkdep executable ------------------------------

AC_DEFUN([ROBOT_PROG_CC], [
AC_REQUIRE([AC_PROG_CC])
    if test "x${GCC}" = "xyes"; then
      CFLAGS="${CFLAGS} -Wall"
    fi
])

AC_DEFUN([ROBOT_PROG_MKDEP],
[
   AC_PATH_PROG(MKDEP, mkdep, no, $exec_prefix/bin:$prefix/bin:$PATH)
   if test "$MKDEP" = "no"; then
      AC_MSG_ERROR([You need the mkdep package])
   fi

   AC_CACHE_CHECK(
      [whether mkdep accepts -I/-D options], ac_cv_robot_mkdep,
      [
         if $MKDEP -I foo -D bar 1>/dev/null 2>&1; then
            ac_cv_robot_mkdep=yes;
         else
            ac_cv_robot_mkdep=no;
         fi
      ])
   if test x"${ac_cv_robot_mkdep}" = xno; then
      AC_MSG_ERROR([You need the mkdep package])
   fi 
])

AC_DEFUN([ROBOT_PROG_PURIFY], [
    opt_purify=no
    AC_ARG_WITH(purify, AC_HELP_STRING([--with-purify], [configure to postprocess with purify]), opt_purify=$withval)
    if test "x$opt_purify" = xyes; then
      AC_MSG_RESULT([enabling purify])
      PURIFY="purify"
    else
      PURIFY=
    fi
    AC_SUBST(PURIFY)
])

AC_DEFUN([AC_CHECK_LIBMATH], [
 ac_sv_libmath_libs=$LIBS
 LIBS=
 AC_SEARCH_LIBS([fabs], [m])
 AC_SEARCH_LIBS([atan], [m])
 LIBMATH_LDFLAGS=$LIBS
 LIBS=$ac_sv_libmath_libs

 AC_SUBST(LIBMATH_LDFLAGS)
])

AC_DEFUN([AC_CHECK_LIBRT], [
 ac_sv_librt_libs=$LIBS
 LIBS=
 AC_SEARCH_LIBS([timer_create], [rt])
 LIBRT_LDFLAGS=$LIBS
 LIBS=$ac_sv_librt_libs

 AC_SUBST(LIBRT_LDFLAGS)
])

dnl --- look for RTAI includes ------------------------------------------
dnl AC_CHECK_RTAI_INCLUDES(var, path)
AC_DEFUN([AC_CHECK_RTAI_INCLUDES],
[
   AC_MSG_CHECKING([for RTAI includes])
   AC_CACHE_VAL(ac_cv_path_rtai,
    [
       IFS="${IFS= 	}"; ac_save_ifs="$IFS"; IFS=":"
        ac_tmppath="$2:/usr/realtime/include:/usr/src/rtai/include"
        for ac_dir in $ac_tmppath; do 
            test -z "$ac_dir" && ac_dir=.
            if eval test -f $ac_dir/rtai.h; then
               eval ac_cv_path_rtai="$ac_dir"
               break
            fi
       done
       IFS="$ac_save_ifs"
    ])
   $1="$ac_cv_path_rtai"
   if test -n "[$]$1"; then
      AC_MSG_RESULT([$]$1)
   else
      AC_MSG_ERROR([cannot find RTAI includes], 2)
   fi
   AC_SUBST($1)
])


dnl --- look for linux kernel includes ----------------------------------
dnl AC_CHECK_LINUXKERNEL_INCLUDES(var, path)
AC_DEFUN([AC_CHECK_LINUXKERNEL_INCLUDES],
[
   AC_MSG_CHECKING([for linux kernel includes])
   AC_CACHE_VAL(ac_cv_path_kernel,
    [
       IFS="${IFS= 	}"; ac_save_ifs="$IFS"; IFS=":"
        ac_tmppath="$2:/usr/realtime/include:/usr/src/linux/include:/usr/include"
        for ac_dir in $ac_tmppath; do 
            test -z "$ac_dir" && ac_dir=.
            if eval test -f $ac_dir/linux/kernel.h; then
               eval ac_cv_path_kernel="$ac_dir"
               break
            fi
       done
       IFS="$ac_save_ifs"
    ])
   $1="$ac_cv_path_kernel"
   if test -n "[$]$1"; then
      AC_MSG_RESULT([$]$1)
   else
      AC_MSG_ERROR([cannot find linux kernel includes], 2)
   fi
   AC_SUBST($1)
])

AC_DEFUN([ROBOT_CHECK_OSAPI], [
    AC_ARG_WITH(rtai,
	       AC_HELP_STRING([--with-rtai=yes|no|lxrt],
		   [configure to use RTAI]),
	       [opt_rtai=$withval],[opt_rtai=no])
    AC_ARG_WITH(rtai-includes,
	       AC_HELP_STRING([--with-rtai-includes=DIR],
		   [RTAI headers are in DIR]),
	       [opt_rtai_includes=$withval])
    AC_ARG_WITH(linux-kernel-includes,
	       AC_HELP_STRING([--with-linux-kernel-includes=DIR],
		   [linux kernel headers are in DIR]),
	       [opt_lnxkern_includes=$withval])

    # make related stuff
    if test "x$opt_rtai" = "xyes"; then
       OSAPI=rtai
       AC_MSG_RESULT([Configuring to use RTAI])
       AC_CHECK_RTAI_INCLUDES(RTAI_INCLUDES, $opt_rtai_includes)
       AC_CHECK_LINUXKERNEL_INCLUDES(LNXKERN_INCLUDES, $opt_lnxkern_includes)

    elif test "x$opt_rtai" = "xlxrt"; then
       OSAPI=lxrt
       AC_DEFINE(__LXRT__, [], [If we are building for LXRT])
       AC_MSG_RESULT([Enabling LXRT userspace support])
       AC_CHECK_RTAI_INCLUDES(RTAI_INCLUDES, $opt_rtai_includes)

    elif test "x$opt_rtai" = "xno"; then :
       OSAPI=posix
       AC_MSG_RESULT([Configuring to use POSIX])
    else
       AC_MSG_ERROR([invalid option --with-rtai=$opt_rtai], 2)
    fi
    AC_SUBST(OSAPI)
])

